// QANav Report Tracker Copyright 2008 Ronald D. Grindle

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import RTObj.ExecRepBld;
import RTObj.TRRepBld;

import com.qanavigation.Basics.RsltString;
import com.qanavigation.QNservices.QNDBacc2;
import com.qanavigation.QNservices.QNDefs;
import com.qanavigation.QNservices.SrvltReqTools;

/**
* Service interface level calls to report QN messages (TC Execution, PTs, etc.)
*/
public class QNRepSrvc extends HttpServlet {

	static final long serialVersionUID = 61940014;
	
	//---------------------------------------------
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		super.service( request, response );
	}

	//---------------------------------------------
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		handleReq( request, response );
	}

	//---------------------------------------------
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		handleReq( request, response );
	}

	//---------------------------------------------
	public void handleReq(HttpServletRequest request, HttpServletResponse response) throws IOException {

		boolean notHandled = true;
		
		QNDBacc2 dBAcc;

		RsltString rsp = new RsltString( true );
		
		String user;
		String QNA;
		
		rsp.setString( QNDefs.c_HTMLMsgMsgHead + "(No processing)" + QNDefs.c_HTMLMsgTail + QNDefs.c_HTMLMsgRepFailHead + "</p>" );
		
		response.setContentType( "text/html" );

		PrintWriter out = response.getWriter();

		//ResourceBundle resBndl = ResourceBundle.getBundle( "QNLabels", request.getLocale() );

		SrvltReqTools sTools = new SrvltReqTools( this, request );
		
		user = request.getRemoteUser();
		
		if( !rsp.checkRslt( sTools ) ) {
			
			rsp.addRsltAddInfo( "Init of SrvltReqTools failed." );
		}

		if( rsp.isOK() ) {
			
			dBAcc = new QNDBacc2();
			
			if( rsp.checkRslt( dBAcc ) ) {
				
				QNA = sTools.getStrVal( "QNA" );

				// manage requests -----------------------------------------
				
				if( QNA.compareTo( "RepPRBI" ) == 0 ) {
		
					notHandled = false;
					
					rsp = RepPRBI( dBAcc, sTools, user );
				}					
			
				if( notHandled ) {
					
					if( QNA.compareTo( "RepTExecBI" ) == 0 ) {
	
						notHandled = false;
						
						rsp = RepTExecBI( dBAcc, sTools, user );
					}
				}

				// unknown request -----------------------------------------
				if( notHandled ) {
	
					rsp.setString( QNDefs.c_HTMLMsgMsgHead + "Wrong QNA '" + QNA + "'." + QNDefs.c_HTMLMsgTail + QNDefs.c_HTMLMsgRepFailHead + "</p>" );
					rsp.setInError( "Wrong QNA '" + QNA + "'." );
				}
			}
			
			dBAcc.close();			
		}
		
		out.println( QNDefs.c_HTMLHead + rsp.getString() + QNDefs.c_HTMLTail );
		
//SrvltReqTools.logErr( this, request, res );		

		if( rsp.isOK() ) {

			// reply is truncated to 59 char (per line?)!!!!
			//out.println( "<p>" + resp + "</p>" );
		}
		else {
			
			//UserMailTool mt = new UserMailTool();
			//String mMsg = "An error occured: " + res.getRsltFinalMsg();
			//String subj = "QANav error at " + request.getRequestURL();
			//mt.admMailToAdm( subj, mMsg, mMsg );
			
			//String errMsg1 = "Transaction is in error.";
			//String errMsg2 = SFormatter.format( "Error reason: {0}", res.getRsltFinalMsg() );
			//String errMsg3 = "The administrator was notified.";

			SrvltReqTools.logErr( this, request, rsp );

			//out.println( "<p>" + errMsg1 + "</p>" );			
			//out.println( "<p>" + errMsg2 + "</p>" );
			//out.println( "<p>" + errMsg3 + "</p>" );
		}
	}
		
	// -----------------------------
	private RsltString RepTExecBI( QNDBacc2 dBAcc, SrvltReqTools sTools, String user ) {
		
		RsltString res = new RsltString( true );
		
		RsltString UID = new RsltString();
		
		String resp;
		
		ExecRepBld exrb = new ExecRepBld( dBAcc.m_DB, sTools, QNDefs.c_RMFilePath );
		
		if( !(res.checkRslt( exrb )) ) {
			
			res.addRsltAddInfo( "\"ExecRepBld()\" failed." );
		}
		else {
			
			exrb.bldExecRep();
			
			if( !(res.checkRslt( exrb )) ) {
				
				res.addRsltAddInfo( "\"bldExecRep()\" failed." );
			}
			else {
		
				UID = exrb.save2DB( );

				if( !(res.checkRslt( UID )) ) {
					
					res.addRsltAddInfo( "\"save2DB()\" failed." );
				}
			}
		}				
	
		if( res.notOK() ) {
			
			resp = QNDefs.c_HTMLMsgRepFailHead + "failed</p>" + QNDefs.c_HTMLMsgMsgHead + "Reporting execution report failed, reason: '" + res.getRsltFinalMsg() + "'" + QNDefs.c_HTMLMsgTail;
			res.addRsltAddInfo( "\"RepTExecBI\" failed." );					
		}
		else {
			
			resp = QNDefs.c_HTMLMsgRepOKHead + "OK</p>" + QNDefs.c_HTMLMsgUIDHead + UID.getString() + QNDefs.c_HTMLMsgTail;
		}
		
		res.setString( resp, false );
		
		return res;	
	}
		
	// -----------------------------
	private RsltString RepPRBI( QNDBacc2 dBAcc, SrvltReqTools sTools, String user ) {
		
		RsltString res = new RsltString( true );
		
		boolean isDraft = false;
		boolean isSub;
		boolean wTicket = false;
		
		RsltString val;
		RsltString UID = new RsltString();
		
		String tcExTitle = "";
		String tcExUid;
		String resp;
		
		TRRepBld prb = new TRRepBld( dBAcc.m_DB, sTools, QNDefs.c_RMFilePath );					
		
		val = sTools.getVal( "ProbStat" );
		
		if( val.getString().compareTo(  "dr"  ) == 0 ) {
			
			isDraft = true;
		}
		
		isSub = !( isDraft );
		
		val = sTools.getVal( "wTicket" );
		
		if( val.getString().compareTo(  "true"  ) == 0 ) {
			
			wTicket = true;
		}
		
		val = sTools.getVal( "TCExUID" );
		
		tcExUid = val.getString();

		if( tcExUid.length() > 0 ) {

			val = prb.m_xDB.getAttrVal( tcExUid, "Cmpgn" );

			if(  val.isOK() ) {

				tcExTitle += val.getString() + ": ";
			}

			val = prb.m_xDB.getAttrVal( tcExUid, "Title" );

			if( val.isOK() ) {

				tcExTitle += val.getString();
			}
			
			if( tcExTitle.length() > 0 ) {
				
				 sTools.add2ParamLst( "TCExTitle", tcExTitle );
			}
		}
	
		if( !(res.checkRslt( prb )) ) {
			
			res.addRsltAddInfo( "\"ProbRepBld()\" failed." );
		}
		else {
							
			prb.bldProbRep( isDraft, isSub );
			
			if( !(res.checkRslt( prb )) ) {
				
				res.addRsltAddInfo( "\"bldProbRep()\" failed." );
			}
			else {
		
				UID = prb.save2DB( user, isDraft, wTicket );

				if( !(res.checkRslt( UID )) ) {
					
					res.addRsltAddInfo( "\"save2DB()\" failed." );
				}
				else if( isSub ) {
					
					String title = "New Document";
					String txt = "A new document was created by Interface";
					
					prb.addLogX( UID.getString(), sTools.m_Req.getRemoteUser(), title, txt, "", "" );
				}
			}
		}
	
		if( res.notOK() ) {
			
			resp = QNDefs.c_HTMLMsgMsgHead + "Reporting a Technical Report failed, reason: '" + res.getRsltFinalMsg() + "'" + QNDefs.c_HTMLMsgTail + QNDefs.c_HTMLMsgRepFailHead + "</p>";
			res.addRsltAddInfo( "\"RepPRBI\" failed." );					
		}
		else {
			
			resp = QNDefs.c_HTMLMsgRepOKHead + "OK</p>" + QNDefs.c_HTMLMsgUIDHead + UID.getString()+  QNDefs.c_HTMLMsgTail;
		}
		
		res.setString( resp, false );
		
		return res;
	}

}
