//---------------------------------------------------------
//piQa Services Collection
//Copyright � 2004 - 2009 Ronald D. Grindle
//---------------------------------------------------------

package RTtools;

import RTObj.FixDocBld;
import RTObj.ShpDocBld;

import com.qanavigation.Basics.Rslt;
import com.qanavigation.Basics.Rslt2DArray;
import com.qanavigation.Basics.RsltLst;
import com.qanavigation.Basics.RsltStatVal;
import com.qanavigation.Basics.RsltString;
import com.qanavigation.Basics.SFormatter;
import com.qanavigation.DBacc.DBacc2;
import com.qanavigation.QNservices.QNDefs;
import com.qanavigation.QNservices.QNDocManSrvc;
import com.qanavigation.QNservices.QNRscBundle;
import com.qanavigation.QNservices.SrvltReqTools;
import com.qanavigation.QNservices.QNDefs.QNDocType;
import com.qanavigation.XMLDB.XMLDB;


/** QN document manipulator.
 * <p>Service class. Do your stuff, check the result at the end.</p> 
 * <p>QNDocMan isA Rslt.</p>
 * @author RDG
 */
public class QNDocMan extends QNDocManSrvc {
	
	private UserMailTool m_uMTool;
	//private UserRoleTool m_uRTool;
	private UserSrvcTool m_uSTool;
	
	private ExtCallHooks m_extHook;
	
	/**
	 * 
	 * @param p_DBAcc
	 * @param p_DocType
	 */
	public QNDocMan( DBacc2 p_DBAcc, String p_DocType ) {
		
		super( p_DBAcc, p_DocType );
		
		initMe();
	} 
		
	/**
	 * Constructor for testing purposes.
	 * @param p_UID
	 * @param p_xDB
	 */		
	public QNDocMan( XMLDB p_xDB, String p_DocType ) {
		
		super( p_xDB, p_DocType );
		
		initMe();
	}
	
	/**
	 * 
	 * @param p_Author
	 * @param p_ShpRel
	 * @param p_ShpDate
	 * @return
	 */
	public ShpDocBld createShipDoc( SrvltReqTools p_SRT, String p_FilePath, String p_Author, String p_Appl, String p_ShpRel, String p_ShpDate, String p_Cmnt ) {
		
		ShpDocBld sdb = new ShpDocBld( m_xDB, p_SRT, p_FilePath );
		
		sdb.bldShpLstDoc(p_Author, p_Appl, p_ShpRel, p_ShpDate, p_Cmnt );
		
		return sdb;
	}
	
	/**
	 * 
	 * @param p_ResBndl
	 * @param p_docUrl
	 * @param p_DocUID
	 * @param p_Assignee
	 * @param p_LogMsg
	 * @param p_WithMail
	 * @return The notification message about sending the mail.
	 */
	public RsltString assignTo( QNRscBundle p_ResBndl, String p_docUrl, String p_DocUID, String p_Assignee, String p_LogMsg, boolean p_WithMail, boolean p_WithLog ) {
		
		Rslt res;
		RsltString rStr = new RsltString( "" );
		
		String dType;
		String msg;
		String pMsg;
		String hMsg;
		String mSubj;
		String ticket;
		String status;
		String statusLbl;
		
		if( isOK() ) {
			
			res = m_xDB.setAttrVal( p_DocUID, QNDefs.c_AttrAssignd, p_Assignee );
			
			if( rStr.checkRslt( res ) ) {
				
				if( p_WithLog ) {
					
					rStr = m_xDB.getAttrVal( p_DocUID, m_StatAttr );
			
					res = appProcLogX( p_DocUID, p_LogMsg, rStr.getString(),  p_Assignee );
				}
				else {
					
					res = appProcLogX( p_DocUID, p_LogMsg );
				}
			}
			
			if( rStr.checkRslt( res ) && p_WithMail ) {
				
				RsltString probTitle = getTitle( p_DocUID );
				
				rStr = m_xDB.getAttrVal( p_DocUID, QNDefs.c_AttrTicket );
				ticket = rStr.getString();
				
				//rStr = m_xDB.getAttrVal( p_DocUID, QNDefs.c_AttrProbStat );
				rStr = getDocStat( p_DocUID );
				status = rStr.getString();
				
				statusLbl = p_ResBndl.getString( "valStat_" + status );
				
				dType = m_xDB.getNodeName( p_DocUID ).getString();
								
				mSubj = SFormatter.format( p_ResBndl.getString( "lblRepAssgn_" + dType ), ticket );

				msg = SFormatter.format( p_ResBndl.getString( "msgRepAssgnForProc" ), ticket, statusLbl );	

				hMsg = "<p>" + msg + "</p>";
				hMsg += "<p>" + p_ResBndl.getString( "msgPlsRefLnk" ) + "</p>";
				hMsg += "<p><a href=\"" + p_docUrl + "\">" + probTitle.getString() + "</a></p>";

				pMsg = msg;
				pMsg += "\n" + p_ResBndl.getString( "msgPlsRefLnk" );
				pMsg += "\n" + p_docUrl + " (" + probTitle.getString() + ")";
			
				rStr = mailToUser( p_ResBndl, p_Assignee, mSubj, hMsg, pMsg );
			}	
		}
		
		return rStr;
	}
	
	// --------------------------------
	protected RsltString getAssn4Rej( String p_DocUID  ) {
		
		RsltString rAssgn;
		
		RsltString rAppl;
		RsltString rRel;
		
		RsltString rAuthor;
		
		rAuthor = getAuthor( p_DocUID );
		
		if( m_DocType == QNDocType.TR  ) {
			
			rAppl = getTRAppl( p_DocUID );
			
			rRel = getSoftRelease( p_DocUID );

			rAssgn = m_uSTool.getAssgn4TRRej( rAuthor.getString(), rAppl.getString(), rRel.getString() );
		}
		else if( m_DocType ==  QNDocType.CR ) {
			
			rAssgn = m_uSTool.getAssgn4CRRej( rAuthor.getString() );
		}
		else if( m_DocType ==  QNDocType.ShpDoc ) {
			
			rAssgn = m_uSTool.getAssgn4ShpRej( rAuthor.getString() );
		}
		else {
			
			rAssgn = new RsltString();
			rAssgn.setInError( "QNDocManSrvc.getAssn4Rej(): Doc type not supported." );
		}
		
		return rAssgn;
	}
	
	/**
	 * 
	 * @param p_UID
	 * @return
	 */
	public ShpDocBld getNewShipDoc( SrvltReqTools p_SRT, String p_FilePath ) {
		
		ShpDocBld sdb = new ShpDocBld( m_xDB, p_SRT, p_FilePath );
		
		sdb.prepShpLstDoc();
		
		return sdb;
	}
	
	/**
	 * 
	 * @param p_DocUID
	 * @param p_ResBndl
	 * @param p_Author
	 * @param p_Ticket
	 * @param p_FixMod
	 * @param p_FixDesc
	 * @param p_FixImp
	 * @param p_FixTstReq
	 * @param p_FixLst
	 * @return
	 */
	public RsltString createFinalFixDoc( String p_DocUID, QNRscBundle p_ResBndl, String p_Author, String p_FixDesc, String p_FixImp, String p_FixTstReq, Rslt2DArray p_FixLst ) {
		
		RsltString resStr;
		//String rec = "";
		
		String status = QNDefs.c_StatFixn;
		String msg;
		
		RsltString probAppl = getAppl( p_DocUID );
		RsltString probFctn = getFctn( p_DocUID );
		RsltString probRelease = getSoftRelease( p_DocUID );
		RsltString probTitle = getTitle( p_DocUID );
		RsltString ticket = getTicket( p_DocUID );
		
		//receiver = getAssignee();
		//rec = receiver.getString();	
		
		/*
		   bNode.addAttribute( "DateTime", fix[0] );
			 bNode.addAttrAsHTML( "Name", fix[1] );
			 bNode.addAttrAsHTML( "Path", fix[2] );
			 bNode.addAttrAsHTML( "Rel", fix[3] );
			 bNode.addAttrAsHTML( "SrcSrv", fix[4] );
		*/
												
		FixDocBld fdb = new FixDocBld( m_xDB );
						
		resStr = fdb.createNewFXdoc( probAppl.getString(), p_Author, ticket.getString(), probFctn.getString(), p_FixDesc, p_FixImp, probRelease.getString(), 
				status, p_FixTstReq, probTitle.getString(), p_FixLst, p_DocUID );
		
		if( resStr.isOK() ) {
			
			msg = SFormatter.format( p_ResBndl.getString( "msgFixDocGen" ), SFormatter.encodeHTML( resStr.getString() ) );

			resStr.checkRslt( appFxDocRef( p_DocUID, resStr.getString(), status, probTitle.getString() ) );

			appProcLogX( p_DocUID, msg );
		}
		
		if( resStr.notOK() ) {
			
			resStr.addRsltAddFailInfo( "QNDocManSrvc", "createFinalFixDoc", "", "" );
		}
		
		return resStr;
	}	
	
	/**
	 * 
	 * @param p_DocUID
	 * @param p_ResBndl
	 * @param p_Author
	 * @param p_FixDesc
	 * @param p_FixImp
	 * @param p_FixTstReq
	 * @param p_FixLst
	 * @return
	 */
	public Rslt updateFixDoc( String p_FixUID, QNRscBundle p_ResBndl, String p_Author, String p_FixDesc, String p_FixImp, String p_FixTstReq, Rslt2DArray p_FixLst ) {
		
		Rslt res;
		
		int i, x;
		
		String msg;
		
		RsltLst<String> trUIDs;
		
		FixDocBld fdb = new FixDocBld( m_xDB );
						
		res = fdb.updFXdoc( p_FixUID, p_Author, p_FixDesc, p_FixImp, p_FixTstReq, p_FixLst );
		
		if( res.isOK() ) {
			
			msg = SFormatter.format( p_ResBndl.getString( "msgFixDocChg" ), p_FixUID );
			
			trUIDs = getTrUIDList4Fix( p_FixUID );
			
			if( res.checkRslt( trUIDs ) ) {
				
				x = trUIDs.size();
				
				for( i = 0; i < x; ++i ) {
					
					appProcLogX( trUIDs.get( i ), msg );					
				}
			}
		}
		else {
			
			res.addRsltAddFailInfo( "QNDocManSrvc", "updateFixDoc", "", "" );
		}
		
		return res;
	}		
	
	/**
	 * Sets the reports status, issues required mails.
	 * <p>Also handles empty status (no action).
	 * <p>Executes only if isOK().
	 * @param p_DocUID
	 * @param p_ResBndl
	 * @param p_NewStat
	 * @param p_docUrl
	 * @param p_User
	 * @return A message about the processing.
	 */
	public RsltString hndlDocStat( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		boolean isNotHndl = true;
		
		RsltString rMsg = new RsltString( "" );
		
		if( p_NewStat.length() > 0 ) {
			
			// -------------------------------			
			//if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatBeta ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = setStatUnAssng( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}				
			//}			

			// -------------------------------
			// Continue (from postponed)
			if( isNotHndl ) {
				if( p_NewStat.compareTo( QNDefs.c_StatCont ) == 0 ) {

					isNotHndl = false;

					rMsg = StatCont( p_DocUID, p_ResBndl, p_User );

//					rMsg.appString( "<p> StatCont <p>" );
				}
			}
			
			// -------------------------------
			// comment requested
			if( isNotHndl ) {
			
				if( p_NewStat.compareTo( QNDefs.c_StatCmntReq ) == 0 ) {
				
					isNotHndl = false;
				
					rMsg = StatCmntReq( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> cmnt req <p>" );				
			}
			
			if( isNotHndl ) {
							
				// estimated.
				if( p_NewStat.compareTo( QNDefs.c_StatEsti ) == 0 ) {
					
					isNotHndl = false;
					
					rMsg = stdStat( p_DocUID, p_ResBndl, p_NewStat, p_User );
//rMsg.appString( "<p> esti <p>" );		
				}
			}
			
			if( isNotHndl ) {
							
				// fixing. Another developer has the chance to add his fix, too
				if( p_NewStat.compareTo( QNDefs.c_StatFixn ) == 0 ) {
					
					isNotHndl = false;					
					
					rMsg = stdStat( p_DocUID, p_ResBndl, p_NewStat, p_User );
				}
//rMsg.appString( "<p> fixn <p>" );				
			}
			
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatProd ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = setStatUnAssng( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}				
			}
			
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatR4Tst ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatR4Tst( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> r4tst <p>" );				
			}
			
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatRej ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatRej( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
				
//rMsg.appString( "<p> rej <p>" );				
			}

			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatRProd ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatRProd( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}			
			}
			
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatRSysTst ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatRSysTst( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}				
			}			
			
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatTstFail ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatTstFail( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> tstfail <p>" );				
			}						

			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatR4Ship ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatR4Ship( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> rship <p>" );				
			}								
			
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatShp ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatShp( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> shp <p>" );				
			}			
		
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatReOpen ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatReOpen( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> reopne <p>" );				
			}			
			
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatClosed ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatClosed( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> closed <p>" );				
			}									
			
			if( isNotHndl ) {
				
				if( p_NewStat.compareTo( QNDefs.c_StatRespGiven ) == 0 ) {
					
					isNotHndl = false;
							
					rMsg = StatRespGiven( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> resp given <p>" );				
			}
			
			if( isNotHndl ) {
				
				if( (p_NewStat.compareTo( QNDefs.c_StatBaWai ) == 0 ) ||
					(p_NewStat.compareTo( QNDefs.c_StatBaWaiOrd ) == 0 ) ||
					(p_NewStat.compareTo( QNDefs.c_StatPostP ) == 0 ) ) {
					
					isNotHndl = false;
							
					rMsg = StatWai( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
				}
//rMsg.appString( "<p> Wai <p>" );				
			}			

			// default handler
			if( isNotHndl ) {
				
				rMsg = stdStat( p_DocUID, p_ResBndl, p_NewStat, p_User );
				
//rMsg.appString( "<p> default <p>" );			
			}
		}
		
		if( rMsg.notOK() ) {
			
			rMsg.addRsltAddInfo( "hndlDocStat(...) failed for '" + p_NewStat + "'" );
			
			rMsg.setString( SFormatter.format( p_ResBndl.getString( "errSetStatFail" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
			
			rMsg.setRsltStat( RsltStatVal.inError );
		}
//rMsg.appString( "<p>hndlDocSts</p>" );		
		return rMsg;	
	}
	
	// -------------------
	private void initMe() {
		
		m_uMTool = new UserMailTool( m_xDB.m_DB );
		m_uSTool = new UserSrvcTool( m_xDB );
		
		m_extHook = new ExtCallHooks( this );
	}
	
	/**
	 * Special request: with copy to Co-coordinator of same group, if co-ordinator.
	 * @param p_ResBndl
	 * @param p_Rec
	 * @param p_Subj
	 * @param p_HMsg
	 * @param p_PMsg
	 * @return The notification message about sending the mail.
	 */
	public RsltString mailToUser( QNRscBundle p_ResBndl, String p_User, String p_Subj, String p_HMsg, String p_PMsg ) {
		
		RsltString rStr = new RsltString( true );
		
		Rslt res;
		
		String sender;
		String senderClear;

		p_HMsg += "<p>" + p_ResBndl.getString( "msgAutoMsg" ) + "</p>";	
		p_HMsg += "<p>" + SrvltReqTools.getAnchPbP() + "</p>";		
		
		p_PMsg += "\n" + p_ResBndl.getString( "msgAutoMsg" );
		p_PMsg += "\n" + SrvltReqTools.getAnchPbP();
		
		p_Subj = SFormatter.decodeHTML( p_Subj );
		p_PMsg = SFormatter.decodeHTML( p_PMsg );
		
		sender = p_ResBndl.getString( "valSenderMail" );	
		senderClear = p_ResBndl.getString( "valSenderClear" );

		res = m_uMTool.mailToUser( p_User, p_Subj, p_HMsg,  p_PMsg, sender, senderClear );

		if( rStr.checkRslt( res ) ) {

			rStr.setString( SFormatter.format( p_ResBndl.getString( "msgNotMsgTo" ), p_User ) );
		}
		else {
			
			rStr.addRsltInitialInfo( "mailToUser() failed." );
		}
		
		return rStr;
	}			
	
	// --------------------------
	protected RsltString reopenCRDoc( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {	
		
		RsltString rMsg = new RsltString( "" );
		
		String assgn = p_User;
	
		String msg;
		
		Rslt res;
		RsltString rStr;
		
		rStr = m_uSTool.getAss4ReoCR();
		
		rStr = stdAssngSetStat( rStr, p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
		
		return rMsg;
	}
	
	// --------------------------------
	protected RsltString reopenDoc( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg;
		
		if( m_DocType.compareTo( QNDocType.TR ) == 0 ) {
			
			rMsg = reopenTRDoc( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
		}
		else if( m_DocType.compareTo( QNDocType.CR ) == 0 ) {
			
			rMsg = reopenCRDoc( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
		}
		else {
			
			rMsg = new RsltString();
			rMsg.setInError( "QNDocManSrvc.reopenDoc(): Doc type not supported." );
		}
		
		return rMsg;
	}	
	
	// --------------------------
	protected RsltString reopenTRDoc( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {	
		
		RsltString rMsg = new RsltString( "" );
		
		String assgn = p_User;
	
		String cStat;
		String nStat;
		String msg;
		
		Rslt res;
		RsltString rStr;
		
		RsltString rAppl;
		RsltString rRel;		
		
		rAppl = getTRAppl( p_DocUID );
		rRel = getSoftRelease( p_DocUID );
		
		rStr = m_uSTool.getAss4ReoTR( rAppl.getString(), rRel.getString() );
		
		if( rMsg.checkRslt( rStr ) ) {
			
			assgn = rStr.getString();
		
			msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );
			rStr = assignTo( p_ResBndl, p_docUrl, p_DocUID, assgn, msg, false, false );
			
			if( rMsg.checkRslt( rStr ) ) {						
	
				nStat = QNDefs.c_StatRej;
				msg = SFormatter.format( p_ResBndl.getString( "msgFixIsSetTo" ), p_ResBndl.getString( "valStat_" + nStat ) );
				
				cStat = getDocStat( p_DocUID ).getString();
				res = setFixStatX( p_DocUID, cStat, nStat, msg );	
				
				if( rMsg.checkRslt( res ) ) {
					
					rMsg.setString( msg );
				}
			}
		}		
// ################# UserMailTool!!!		
		
		msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
		
		res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );
		
		if( rMsg.checkRslt( res ) ) {
			
			rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
		}
		
		return rMsg;
	}
	
	
	// ----------------------
	private RsltString setStatUnAssng( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( true );

		Rslt res;

		String assgn = "";
		String msg;

		msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );

		rMsg = assignTo( p_ResBndl, p_docUrl, p_DocUID, assgn, msg, false, false );

		if( rMsg.isOK() ) {

			msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );

			res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );

			rMsg.checkRslt( res );
		}

		if( rMsg.isOK() ) {

			rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );

			m_extHook.statChg( p_DocUID, p_NewStat, "" );
		}			

		return rMsg;
	}
	
	// -----------------------	
	private RsltString StatClosed( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );
		
		String assgn = p_User;
		
		String cStat;

		String msg;
		
		RsltString rStr;
		
		Rslt res;
		
		msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), "", QNDefs.c_UserSys );
		
		assgn = "";
		
		// Assign back to the author?
		rStr = assignTo( p_ResBndl, p_docUrl, p_DocUID, assgn, msg, false, false );	
		
		if( rMsg.checkRslt( rStr ) && ( m_DocType.compareTo( QNDocType.TR ) == 0 ) ) {

			msg = SFormatter.format( p_ResBndl.getString( "msgFixIsSetTo" ), p_ResBndl.getString( "valStat_" + p_NewStat ) );
			
			cStat = getDocStat( p_DocUID ).getString();
			res = setFixStatX( p_DocUID, cStat, p_NewStat, msg );
			
			if( rMsg.checkRslt( res ) ) {
				
				rMsg.setString( msg );
			}
		}
					
// ################# Move to UserMailTool, lez them decide
		/*
		RsltString probTitle = getTitle( p_DocUID );
		
		mSubj = p_ResBndl.getString( "lblRepReOp" );

		hMsg = "<p>" + p_ResBndl.getString( "msgARepReOpd" ) + "</p>";
		hMsg += "<p>" + p_ResBndl.getString( "msgPlsRefLnk" ) + "</p>";
		hMsg += "<p><a href=\"" + p_docUrl + "\">" + probTitle.getString() + "</a></p>";

		pMsg = p_ResBndl.getString( "msgARepReOpd" );
		pMsg += "\n" + p_ResBndl.getString( "msgPlsRefLnk" );
		pMsg += "\n" + p_docUrl + " (" + probTitle.getString() + ")";
		
		rStr = mailToUser( p_ResBndl, rec, mSubj, hMsg, pMsg );
				
				if( rMsg.checkRslt( rStr ) ) {

			rMsg.appString( rStr.getString() );
		}
		*/		
		
		msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
		
		res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );
		
		if( rMsg.checkRslt( res ) ) {
		
			m_extHook.statChg( p_DocUID, p_NewStat, "" );
			
			rMsg.appString( " " + msg );
		}
		
		return rMsg;
	}	
	
	// -----------------
	private RsltString StatCmntReq( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( p_ResBndl.getString( "msgReqComIss" ) );
		
		String pMsg;
		String hMsg;
		String mSubj;
		String nStat;
		String msg;
		String pTitle;
		
		RsltString rStr;
		
		Rslt res;

		String assgn = p_User;

		rStr = getTitle( p_DocUID ); 
		
		if( rMsg.checkRslt( rStr ) ) {
			
			pTitle = rStr.getString();
		
			rStr = getAuthor( p_DocUID );
			
			if( rMsg.checkRslt( rStr ) ) {						
		
				assgn = rStr.getString();
				
				res = m_xDB.setAttrVal( p_DocUID, QNDefs.c_AttrReqstr, p_User );
	
				rStr = m_xDB.getAttrVal( p_DocUID, QNDefs.c_AttrProbStat  );
				nStat = rStr.getString();					
				res = m_xDB.setAttrVal( p_DocUID, QNDefs.c_AttrProbProcStat, nStat );
	
				if( rMsg.checkRslt( res ) ) {	
				
					msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, p_User );
	
					rStr = assignTo( p_ResBndl, "", p_DocUID, assgn, msg, false, false );
					
					if( rMsg.checkRslt( rStr ) ) {
						
// ################# Move to UserMailTool						
		
						mSubj = p_ResBndl.getString( "lblCommReq" );
						
						rStr = m_xDB.getAttrVal( p_DocUID, QNDefs.c_AttrTicket );
						
						String ycr = SFormatter.format( p_ResBndl.getString( "msgYCmntReq" ), rStr.getString() );
						String prl = p_ResBndl.getString( "msgPlsRefLnk" ) ;
						
						hMsg = "<p>" + ycr + "</p>";
						hMsg += "<p>" + prl + "</p>";
						hMsg += "<p><a href=\"" + p_docUrl + "\">" + pTitle + "</a></p>";
	
						pMsg = ycr;
						pMsg += "\n" + prl;
						pMsg += "\n" + p_docUrl + " (" + pTitle + ")";
											
						rStr = mailToUser( p_ResBndl, assgn, mSubj, hMsg, pMsg );
						
						if( rMsg.checkRslt( rStr ) ) {
	
							rMsg.appString( " " + rStr.getString() );
						}
						
						msg = SFormatter.format( p_ResBndl.getString( "msgReqComBy" ), p_User );
						
						appProcLogX( p_DocUID, msg );
					}
				}
			}
		}
		
		msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
		
		res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );
		
		if( rMsg.checkRslt( res ) ) {
			
			m_extHook.statChg( p_DocUID, p_NewStat, "" );
		}
		
		return rMsg;
	}
	
	// -----------------
	private RsltString StatCont( String p_DocUID, QNRscBundle p_ResBndl, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );

		String msg;
		String procStat;
		
		RsltString rStr;
		
		Rslt res;

		rStr = getDocProcStat( p_DocUID );
		
		procStat = rStr.getString();
	
		if( rMsg.checkRslt( rStr ) ) {
		
			msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + procStat ), p_User );
		
			res = setDocStatX( p_DocUID, rStr.getString(), p_User, msg );
	
			if( rMsg.checkRslt( res ) ) {
				
				res = setDocProcStat( p_DocUID, "" );
				
				rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + procStat ) ) );
			}			
		}
		
//rMsg.appString( " <cont>" ); 		
		return rMsg;
	}	
	
	// -----------------------	
	private RsltString StatR4Ship( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );
		
		String assgn = p_User;
		
		String cStat;
		String pMsg;
		String mSubj;
		String msg;
		String pTitle;
		String sender;
		String senderClear;

		RsltString rStr;
		
		Rslt res;
		
		String lblStat;
		
// ################# UserSrvcTool!!!		
		rStr = m_uSTool.getSuper4Dpl();
		
		if( rMsg.checkRslt( rStr ) ) {

			assgn = rStr.getString();
				
			msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );
			
			rStr = assignTo( p_ResBndl, p_docUrl, p_DocUID, assgn, msg, false, false );	
			
			if( rMsg.checkRslt( rStr ) ) {
				
				lblStat = p_ResBndl.getString( "valStat_" + p_NewStat );
				
				if( m_DocType.compareTo( QNDocType.TR ) == 0 ) {

					msg = SFormatter.format( p_ResBndl.getString( "msgFixIsSetTo" ), lblStat );

					cStat = getDocStat( p_DocUID ).getString();
					res = setFixStatX( p_DocUID, cStat, p_NewStat, msg );
					
					rMsg.checkRslt( res );
				}
				
				// e-Mail notification to supervisor
				if( rMsg.isOK() ) {
// ################# Move to UserMailTool
					rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgRepAssgnTo" ), assgn ) );

					rStr = getTicket( p_DocUID );
					String ticket = rStr.getString();
						
					rStr = getTitle( p_DocUID );	
					pTitle = rStr.getString();
					
					mSubj = SFormatter.format( p_ResBndl.getString( "lblPRIsInStat" ), ticket, lblStat );

					String mMsg = "<p>" + mSubj + ":</p>";
					mMsg += "<p><a href=\"" + p_docUrl + "\">" + " " + pTitle + "</a></p>";

					pMsg = mSubj;
					pMsg += "\n" + p_docUrl + " (" + pTitle + ")";
					
					sender = p_ResBndl.getString( "valSenderMail" );	
					senderClear = p_ResBndl.getString( "valSenderClear" );					

					m_uMTool.mail4R2Ship( mSubj, pMsg, mMsg, sender, senderClear );							
				}
			}
		}
		
		msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
		
		res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );
		
		if( rMsg.checkRslt( res ) ) {
			
			rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
			
			m_extHook.statChg( p_DocUID, p_NewStat, "" );			
		}
		
		return rMsg;
	}
		
	// ---------------------------
	private RsltString StatR4Tst( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );
		
		String msg;

		Rslt res;
		RsltString rStr;
		
		String assgn = p_User;
		
		String cStat;
		String lblStat;
		
		RsltString rAppl;
		RsltString rRel;	
		
		rAppl = getTRAppl( p_DocUID );
		rRel = getSoftRelease( p_DocUID );
		
		rStr = m_uSTool.getSuper4TR( rAppl.getString(), rRel.getString() );
		
		if( rMsg.checkRslt( rStr ) ) {
			
			assgn = rStr.getString();
			
			msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );
			
			rStr = assignTo( p_ResBndl, p_docUrl, p_DocUID, assgn, msg, false, false );
			
			if( rMsg.checkRslt( rStr ) ) {
				
				if( m_DocType.compareTo( QNDocType.TR ) == 0 ) {
					
					lblStat = p_ResBndl.getString( "valStat_" + p_NewStat );

					msg = SFormatter.format( p_ResBndl.getString( "msgFixIsSetTo" ), lblStat );

					cStat = getDocStat( p_DocUID ).getString();
					res = setFixStatX( p_DocUID, cStat, p_NewStat, msg );	
				}

// ################# UserMailTool!!!				
			
				rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgRepAssgnTo" ), assgn ) );
			}
		}
		
		msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
		
		res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );
		
		if( rMsg.checkRslt( res ) ) {
			
			rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
			
			m_extHook.statChg( p_DocUID, p_NewStat, "" );			
		}
		
		return rMsg;
	}
	
	// -----------------------
	private RsltString StatRej( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {

		RsltString rMsg = new RsltString( "" );

		String assgn;
		String cStat;
		String pMsg;
		String hMsg;
		String lblStat;
		String mSubj;
		String msg;

		String txt;

		Rslt res;
		RsltString rStr;

		rStr = getAssn4Rej( p_DocUID );

		if( rMsg.checkRslt( rStr ) ) {

			assgn = rStr.getString();

//			################# Move to UserMailTool

			msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );

			rStr = assignTo( p_ResBndl, p_docUrl, p_DocUID,  assgn, msg, false, false );

			if( rMsg.checkRslt( rStr ) ) {

				rMsg.setString( msg );

				if( m_DocType.compareTo( QNDocType.TR ) == 0 ) {

					lblStat = p_ResBndl.getString( "valStat_" + p_NewStat );

					msg = SFormatter.format( p_ResBndl.getString( "msgFixIsSetTo" ), lblStat );

					cStat = getDocStat( p_DocUID ).getString();
					res = setFixStatX( p_DocUID, cStat, p_NewStat, msg );
				}

				if( assgn.length() > 0 ) {
					
					// e mail
					mSubj = p_ResBndl.getString( "lblPRRejctd" );

					RsltString probTitle = getTitle( p_DocUID );

					txt = SFormatter.format( p_ResBndl.getString( "msgPRRejctd" ), getDocNum( p_DocUID ).getString() );

					hMsg = "<p>" + txt + "</p>";
					hMsg += "<p>" + p_ResBndl.getString( "msgPlsRefLnk" ) + "</p>";
					hMsg += "<p><a href=\"" + p_docUrl + "\">" + probTitle.getString() + "</a></p>";
					hMsg += "<p>" + p_ResBndl.getString( "msgIfOKCLose" ) + "</p>";

					pMsg = txt;
					pMsg += "\n" + p_ResBndl.getString( "msgPlsRefLnk" );
					pMsg += "\n" + p_docUrl + " (" + probTitle.getString() + ")";
					pMsg += "\n" + p_ResBndl.getString( "msgIfOKCLose" );

					rStr = mailToUser( p_ResBndl, assgn, mSubj, hMsg, pMsg );

					if( rMsg.checkRslt( rStr ) ) {

						rMsg.appString( rStr.getString() );
					}
				}
			}

			msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );

			res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );

			if( rMsg.checkRslt( res ) ) {

				rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );

				m_extHook.statChg( p_DocUID, p_NewStat, "" );				
			}
		}

		return rMsg;
	}
	
	// -----------------------	
	private RsltString StatReOpen( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = reopenDoc( p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
		
		if( rMsg.isOK() ) {
			
			m_extHook.statChg( p_DocUID, p_NewStat, "" );
		}
		
		return rMsg;
	}
	
	// -----------------------	
	private RsltString StatRespGiven( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );
		
		String assgn = p_User;
		
		String pMsg;
		String hMsg;
		String mSubj;
		String aStat;
		String nStat;
		String msg;
		
		RsltString rStr;
		
		Rslt res;
		
		nStat = p_NewStat;
		
		rStr = getReqstr( p_DocUID );
		
		if( rMsg.checkRslt( rStr ) ) {
		
			assgn = rStr.getString();
			
			rStr = m_xDB.getAttrVal( p_DocUID, QNDefs.c_AttrProbProcStat  );
			aStat =  rStr.getString();
			
			if( aStat.length() > 0 ) {
				
				nStat = aStat; // p_NewStat is set to a new value!!!!
			}
		
			res = m_xDB.setAttrVal( p_DocUID, QNDefs.c_AttrReqstr, "" );
			res = m_xDB.setAttrVal( p_DocUID, QNDefs.c_AttrProbProcStat, "" );
			
			
			if( rMsg.checkRslt( res ) ) {	
			
				msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );

				rStr = assignTo( p_ResBndl, "", p_DocUID, assgn, msg, false, false );
				
				if( rMsg.checkRslt( rStr ) ) {

					rStr = getTitle( p_DocUID );
				}

				if( rMsg.checkRslt( rStr ) ) {
					
					//rMsg.setString( p_ResBndl.getString( "msgYCrHasResp" ) );
// ################# UserMailTool!!!						
					mSubj = p_ResBndl.getString( "lblResponse" );	
					
					hMsg = "<p>" + p_ResBndl.getString( "msgYCrHasResp" ) + "</p>";
					hMsg += "<p>" + p_ResBndl.getString( "msgPlsRefLnk" ) + "</p>";
					hMsg += "<p><a href=\"" + p_docUrl + "\">" + rStr.getString() + "</a></p>";
					
					pMsg = p_ResBndl.getString( "msgYCrHasResp" );
					pMsg += "\n" + p_ResBndl.getString( "msgPlsRefLnk" );
					pMsg += "\n" + p_docUrl + " (" + rStr.getString() + ")";

					rStr = mailToUser( p_ResBndl, assgn, mSubj, hMsg, pMsg );
					
					if( rMsg.checkRslt( rStr ) ) {

						rMsg.appString( rStr.getString() );
					}
					
					msg = SFormatter.format( p_ResBndl.getString( "msgRespGivBy" ), p_User );
					appProcLogX( p_DocUID, msg );								
				}
			}
		}		
		
		msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + nStat ), p_User );
		
		res = setDocStatX( p_DocUID, nStat, assgn, msg );
		
		if( rMsg.checkRslt( res ) ) {
			
			rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
			
			m_extHook.statChg( p_DocUID, p_NewStat, "" );			
		}
		
		return rMsg;
	}
	
	/**
	 * 
	 * @param p_DocUID
	 * @param p_ResBndl
	 * @param p_NewStat
	 * @param p_docUrl
	 * @param p_User
	 * @return
	 */
	private RsltString StatRProd( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {

		RsltString rStr;
				
		rStr = m_uSTool.getSuper4Prod();
		
		rStr = stdAssngSetStat( rStr, p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
		
		return rStr;
	}
		
	/**
	 * 
	 * @param p_DocUID
	 * @param p_ResBndl
	 * @param p_NewStat
	 * @param p_docUrl
	 * @param p_User
	 * @return
	 */
	private RsltString StatRSysTst( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rStr;
		
// ################# UserSrvcTool!!!		
		rStr = m_uSTool.getSuper4SysTst();
		
		rStr = stdAssngSetStat( rStr, p_DocUID, p_ResBndl, p_NewStat, p_docUrl, p_User );
		
		return rStr;
	}
	
	// -----------------------	
	private RsltString StatShp( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );
		
		String assgn = p_User;

		String cStat;
		String lblStat;
		String msg;
		
		Rslt res;
		RsltString rStr;
		
		rStr = getAuthor( p_DocUID );
		
		if( rMsg.checkRslt( rStr ) ) {
		
			assgn = rStr.getString();
		
			msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );
		
			rStr = assignTo( p_ResBndl, p_docUrl, p_DocUID, assgn, msg, false, false );
		
			if( rMsg.checkRslt( rStr ) && ( m_DocType.compareTo( QNDocType.TR ) == 0 ) ) {
				
				lblStat = p_ResBndl.getString( "valStat_" + p_NewStat );
				
				msg = SFormatter.format( p_ResBndl.getString( "msgFixIsSetTo" ), lblStat );
			
				cStat = getDocStat( p_DocUID ).getString();
				res = setFixStatX( p_DocUID, cStat, p_NewStat, msg );				
				
				rMsg.setString( msg );
			}
		}
		
// ################# UserMailTool!!!				
		/*
		RsltString probTitle = getTitle( p_DocUID ); 
		mSubj = p_ResBndl.getString( "lblSolShpd" );

		hMsg = "<p>" + p_ResBndl.getString( "msgAsolShpd" ) + "</p>";
		hMsg += "<p>" + p_ResBndl.getString( "msgPlsRefLnk" ) + "</p>";
		hMsg += "<p><a href=\"" + p_docUrl + "\">" + probTitle.getString() + "</a></p>";
		hMsg += "<p>" + p_ResBndl.getString( "msgIfOKCLose" ) + "</p>";

		pMsg = p_ResBndl.getString( "msgAsolShpd" );
		pMsg += "\n" + p_ResBndl.getString( "msgPlsRefLnk" );
		pMsg += "\n" + p_docUrl + " (" + probTitle.getString() + ")";
		pMsg += "\n" + p_ResBndl.getString( "msgIfOKCLose" );
		
		rStr = mailToUser( p_ResBndl, rec, mSubj, hMsg, pMsg );
				
		if( rMsg.checkRslt( rStr ) ) {

			rMsg.appString( rStr.getString() );
		}
		*/	
		
		
		msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
		
		res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );
		
		if( rMsg.checkRslt( res ) ) {
			
			rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
			
			m_extHook.statChg( p_DocUID, p_NewStat, "" );
			
			rMsg.appString( msg );
		}
		
		return rMsg;
	}

	// -----------------------	
	private RsltString StatTstFail( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );
		
		String assgn = p_User;

		String cStat;
		String lblStat;
		String msg;
		
		RsltString rStr;
		
		Rslt res;
		
		RsltString rAppl;
		RsltString rRel;	
		
		rAppl = getTRAppl( p_DocUID );
		rRel = getSoftRelease( p_DocUID );
		
		rStr = m_uSTool.getSuper4TR( rAppl.getString(), rRel.getString() );
		
		if( rMsg.checkRslt( rStr ) ) {
			
			assgn = rStr.getString();

			msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );
			
			res = assignTo( p_ResBndl, p_docUrl, p_DocUID, assgn, msg, false, false );
			
			if( rMsg.checkRslt( res ) ) {
				
				if( m_DocType.compareTo( QNDocType.TR ) == 0 ) {

					lblStat = p_ResBndl.getString( "valStat_" + p_NewStat );

					msg = SFormatter.format( p_ResBndl.getString( "msgFixIsSetTo" ), lblStat );

					cStat = getDocStat( p_DocUID ).getString();
					res = setFixStatX( p_DocUID, cStat, p_NewStat, msg );
				}
				
				if( rMsg.checkRslt( res ) ) {							
				
					rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgRepAssgnTo" ), assgn ) );
				}
			}
		}
		
// ################# UserMailTool!!!		
		msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
		
		res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );
		
		if( rMsg.checkRslt( res ) ) {
			
			rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
			
			m_extHook.statChg( p_DocUID, p_NewStat, "" );
		}
		
		return rMsg;
	}
	
	// -----------------
	private RsltString StatWai( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );

		String msg;
		
		RsltString rStr;
		
		Rslt res;

		rStr = getDocStat( p_DocUID );
		
//rMsg.appString( rStr.getRsltFinalMsg() );	
		
// ################# UserMailTool!!!				
		
		if( rMsg.checkRslt( rStr ) ) {
			
			res = setDocProcStat( p_DocUID, rStr.getString() );
			
//rMsg.appString( res.getRsltFinalMsg() );			
		
			msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
		
			res = setDocStatX( p_DocUID, p_NewStat, p_User, msg );
	
			if( rMsg.checkRslt( res ) ) {
				
				rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
				
				m_extHook.statChg( p_DocUID, p_NewStat, "" );
			}
		}
		
//rMsg.appString( " StatWai old Stat '" + rStr.getString() + "'" );
		
		return rMsg;
	}
	
	
	// ----------------------
	private RsltString stdAssngSetStat( RsltString p_rStr, String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_docUrl, String p_User ) {
		
		RsltString rMsg = new RsltString( true );
		
		Rslt res;
		
		String assgn;
		String msg;
		
		if( rMsg.checkRslt( p_rStr ) ) {

			assgn = p_rStr.getString();

			msg = SFormatter.format( p_ResBndl.getString( "msgInstAsgnToBy" ), assgn, QNDefs.c_UserSys );

			rMsg = assignTo( p_ResBndl, p_docUrl, p_DocUID, assgn, msg, false, false );

			if( rMsg.isOK() ) {

				msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );

				res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );

				rMsg.checkRslt( res );
			}

			if( rMsg.isOK() ) {

				rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );

				m_extHook.statChg( p_DocUID, p_NewStat, "" );
			}			
		}		
		
		return rMsg;
	}	
	
	// ----------------
	private RsltString stdStat( String p_DocUID, QNRscBundle p_ResBndl, String p_NewStat, String p_User ) {
		
		RsltString rMsg = new RsltString( "" );
		
		String cStat;
		String lblStat;
		String msg;
		
		Rslt res;
		
		String assgn = p_User;
		
		if( m_DocType.compareTo( QNDocType.TR ) == 0 ) {
			
			cStat = getDocStat( p_DocUID ).getString();

			lblStat = p_ResBndl.getString( "valStat_" + p_NewStat );

			msg = SFormatter.format( p_ResBndl.getString( "msgFixIsSetTo" ), lblStat );

			res = setFixStatX( p_DocUID, cStat, p_NewStat, msg );
			
			rMsg.checkRslt( res );
		}
		
		if( rMsg.isOK() ) {
		
			msg = SFormatter.format( p_ResBndl.getString( "msgStatChgToBy" ), p_ResBndl.getString( "valStat_" + p_NewStat ), p_User );
			
			res = setDocStatX( p_DocUID, p_NewStat, assgn, msg );
			
			rMsg.checkRslt( res );
		}
		
		if( rMsg.isOK() ) {
			
			rMsg.setString( SFormatter.format( p_ResBndl.getString( "msgStatSet" ), p_ResBndl.getString( "valStat_" + p_NewStat ) ) );
			
			m_extHook.statChg( p_DocUID, p_NewStat, "" );
		}
//rMsg.appString( "<p>prepStat</p>" );
		return rMsg;
	}
		
}
